/*
 * gsampler-channel.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <glib/gi18n-lib.h>

#include "gsampler-channel.h"
#include "gsampler-utils.h"
#include "gsampler-window.h"
#include "gsampler-option.h"
#include "gsampler-channel-dialog.h"
#include "gsampler-client-device.h"
#include "gsampler-client-midimap.h"

G_DEFINE_TYPE (GsamplerChannel, gsampler_channel, GTK_TYPE_FRAME);

enum {
	CHANNEL_UPDATE_EVENT,
	CHANNEL_NB_SIGNALS
};

static guint channel_signals[CHANNEL_NB_SIGNALS] = { 0 };

static gboolean 
gsampler_channel_init_instrument (GsamplerChannel* self) 
{
	GSList* inst_list = NULL;
	g_return_val_if_fail (self != NULL, FALSE);

 	if (!gsampler_instrument_load_file (self->priv->instrument, self->priv->info->file)) 
	{
		gsampler_main_window_send_message ("warning", g_strconcat ("The file ", self->priv->info->file,
		                                                           " does not exist", NULL));
		return FALSE;
	}
	gsampler_text_combo_remove_all (self->priv->inst_combo);
	inst_list = gsampler_instrument_get_list (self->priv->instrument);
	while (inst_list) 
	{
		struct Struct_list_widget* instru;
		instru = (struct Struct_list_widget*) inst_list->data;
		gsampler_text_combo_insert (self->priv->inst_combo, (const gchar*) instru->Sdata, instru->Sid);
		inst_list = inst_list->next;
	}
	gtk_combo_box_set_active ((GtkComboBox*) self->priv->inst_combo, self->priv->info->instrument);
	return TRUE;
}

static GsamplerChannelInfo* 
gsampler_channel_get_channel_info (GsamplerChannel* self, gint id) 
{
	GsamplerClient* client = NULL;
	GsamplerChannelInfo *new_info;
	g_return_val_if_fail (GSAMPLER_IS_CHANNEL (self), NULL);
	
	client = gsampler_client_get_instance ();
	new_info = gsampler_client_get_channel_info (client, id);
	if (new_info->id == (-1)) 
	{
		gsampler_channel_info_free (new_info);
		g_object_unref (client);
		return NULL;
	}
	g_object_unref (client);
	return new_info;
}

static void 
gsampler_channel_on_channel_delete (GtkButton* button, gpointer data) 
{
	GsamplerClient* client = NULL;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (button != NULL);
	
	client = gsampler_client_get_instance ();
	if (!gsampler_client_delete_channel (client, self->priv->info->id)) 
	{
		gsampler_main_window_send_message ("warning", _("Can't delete channel "));
	}
	g_object_unref (client);
}

static void 
gsampler_channel_on_volume_changed (GtkAdjustment* adj, gpointer data) 
{
	GsamplerClient* client = NULL;
	gdouble vol;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (adj != NULL);
	
	client = gsampler_client_get_instance ();
	vol = gtk_adjustment_get_value (adj);
	if (!gsampler_client_set_channel_volume (client, self->priv->info->id, vol)) 
	{
		gsampler_main_window_send_message ("warning", _("Can't set channel volume "));
	}
	g_object_unref (client);
}


static void 
gsampler_channel_on_audio_changed (GtkComboBox* combo, gpointer data) 
{
	GsamplerClient* client = NULL;
	gint audio_id;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (combo != NULL);
	
	audio_id = gtk_combo_box_get_active (combo);
	if (audio_id == (-1) || audio_id == self->priv->info->audio) return;

	client = gsampler_client_get_instance ();
	if (!gsampler_client_set_channel_output (client, self->priv->info->id, audio_id)) {
		gsampler_main_window_send_message ("warning", _("Can't set channel audio output..."));
	} else {
		gsampler_channel_info_destroy (self->priv->info);
		self->priv->info = gsampler_channel_get_channel_info (self, self->priv->info->id);
	}
	g_object_unref (client);
}

static void
gsampler_channel_update_midi_port_combo (GsamplerChannel* self)
{
	GsamplerClient *client = NULL;
	g_return_if_fail (self != NULL);

	gsampler_text_combo_remove_all (self->priv->port_combo);
	client = gsampler_client_get_instance ();
	GsamplerDeviceInfo *info = gsampler_client_get_device_info (client, self->priv->info->midi, 1);
	gint i;
	for (i = 0; i < info->param; i++)
	{
		gsampler_text_combo_insert (self->priv->port_combo, g_strdup_printf ("Port %d", i), i);
	}
	gtk_combo_box_set_active (self->priv->port_combo, self->priv->info->midi_port);
	gsampler_device_info_free (info);
	g_object_unref (client);

}

static void 
gsampler_channel_on_midi_changed (GtkComboBox* combo, gpointer data) 
{
	GsamplerClient* client = NULL;
	gint midi_id;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (combo != NULL);
	
	midi_id = gtk_combo_box_get_active (combo);
	if (midi_id == (-1) ||  midi_id == self->priv->info->midi) return;

	client = gsampler_client_get_instance ();
	if (!gsampler_client_set_channel_input (client, self->priv->info->id, midi_id)) 
	{
		gsampler_main_window_send_message ("warning", _("Can't set channel midi input..."));
	} else 
	{
		gsampler_channel_info_destroy (self->priv->info);
		self->priv->info = gsampler_channel_get_channel_info (self, self->priv->info->id);
		gsampler_channel_update_midi_port_combo (self);
	}
	g_object_unref (client);
}

static void 
gsampler_channel_on_edit_clicked (GtkButton* button, gpointer data) 
{
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (button != NULL);
	
	if (g_strcmp0 (self->priv->info->engine, "GIG") == 0) 
	{
		GsamplerClient* client = NULL;
		client = gsampler_client_get_instance ();
		if (!gsampler_client_edit_channel_instrument (client, self->priv->info->id)) 
		{
			gsampler_main_window_send_message ("warning", 
			                                   _("Can't launch Gigedit verify your's setup..."));
		}
		g_object_unref (client);
	} else 
	{
		gsampler_main_window_send_message ("warning", 
		                                   _("Can't launch Gigedit for non GIG file..."));
	}
}

static void 
gsampler_channel_on_load_clicked (GsamplerChannel *self) 
{
	GsamplerChannelDialog* dialog = NULL;
	GsamplerClient* client;
	GsamplerMainWindow* win;
	gchar** engine_list;
	gint response;
	g_return_if_fail (self != NULL);
	
	client = gsampler_client_get_instance ();
	win = gsampler_main_window_get_instance ();
	dialog = gsampler_channel_dialog_new ();
	gtk_window_set_transient_for ((GtkWindow*) dialog, (GtkWindow*) win);
	engine_list = gsampler_client_get_available_engine_list (client);
	gint element_it;
	gint engine_list_lenght = g_strv_length (engine_list);
	for (element_it = 0; element_it < engine_list_lenght; element_it = element_it + 1) 
	{
		gchar* element;
		element = g_strdup (engine_list[element_it]);
		gsampler_text_combo_append (dialog->engine, string_replace (element, "'", ""));
		g_free (element);
	}
	gtk_combo_box_set_active ((GtkComboBox*) dialog->engine, 0);
	response = gtk_dialog_run ((GtkDialog*) dialog);
	switch (response) 
	{
		case GTK_RESPONSE_APPLY:
			gsampler_main_window_set_wait_cursor (TRUE);

#ifdef G_OS_WIN32
			self->priv->info->file = string_replace (gtk_file_chooser_get_filename ((GtkFileChooser*) dialog->file), "\\", "/");
#else
			self->priv->info->file = gtk_file_chooser_get_filename ((GtkFileChooser*) dialog->file);
#endif
			self->priv->info->instrument = gtk_combo_box_get_active ((GtkComboBox*) dialog->inst);
			self->priv->info->engine = gsampler_text_combo_get_active (dialog->engine);
			if (!gsampler_client_load_engine (client, self->priv->info->engine, 
			                                  self->priv->info->id)) 
			{
				gsampler_main_window_set_wait_cursor (FALSE);
				gsampler_main_window_send_message ("warning", _("Error can't load engine"));
			} else if (!gsampler_client_load_instrument (client, self->priv->info->id, 
			                                             self->priv->info->file, 
			                                             self->priv->info->instrument)) 
			{
				gsampler_main_window_set_wait_cursor (FALSE);
				gsampler_main_window_send_message ("warning", _("Error can't load instrument"));
			}
			break;
		case GTK_RESPONSE_CANCEL:
			break;
		default:
			break;
	}
	gtk_widget_destroy ((GtkWidget*) dialog);
	g_strfreev (engine_list);
	g_object_unref (win);
	g_object_unref (client);
}

static void
gsampler_channel_on_map_changed (GtkComboBox* combo, gpointer data) 
{
	GtkTreeModel *model = NULL;
	GtkTreeIter iter;
	gint id;
	GsamplerClient* client;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (combo != NULL);

	client = gsampler_client_get_instance ();
	model = gtk_combo_box_get_model (combo);
	if (gtk_combo_box_get_active_iter (combo, &iter)) 
	{
		gtk_tree_model_get (model, & iter, 0, &id, -1);
		if (!gsampler_client_set_channel_midimap (client, self->priv->info->id, id))
			gsampler_main_window_send_message ("warning", _("Can't set midi map..."));
	}
	g_object_unref (client);
}

static void 
gsampler_channel_on_instrument_changed (GtkComboBox* combo, gpointer data) 
{
	GsamplerClient* client;
	gint inst_id;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (combo != NULL);
	
	client = gsampler_client_get_instance ();
	inst_id = gtk_combo_box_get_active (combo);
	if (inst_id != (-1) && inst_id != self->priv->info->instrument && 
	    self->priv->info->inst_status == 100) 
	{
		if (!gsampler_client_load_instrument (client, self->priv->info->id, 
		                                      self->priv->info->file, inst_id)) 
		{
			gsampler_main_window_send_message ("warning", _("Can't launch instrument..."));
		}
	}
	g_object_unref (client);
}

static void 
gsampler_channel_on_midi_channel_changed (GtkComboBox* combo, gpointer data) 
{
	GsamplerClient* client;
	gint chan_id = 0;
	gchar* chan = NULL;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (combo != NULL);
	
	client = gsampler_client_get_instance ();
	chan_id = gtk_combo_box_get_active (combo);
	if (chan_id >= 16 || chan_id == -1) 
	{
		chan = g_strdup ("ALL");
	} else 
	{
		chan = g_strdup_printf ("%i", chan_id);
	}
	if (g_strcmp0 (self->priv->info->midi_chan, chan) != 0)
	{
		if (!gsampler_client_set_channel_midi_channel (client, self->priv->info->id, chan)) 
		{
			gsampler_main_window_send_message ("warning", _("Can't set midi channel channel..."));
		}
	}
	g_free (chan);
	g_object_unref (client);
}

static void 
gsampler_channel_on_midi_port_changed (GtkComboBox* combo, gpointer data) 
{
	GsamplerClient* client;
	gint port;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (combo != NULL);
	
	client = gsampler_client_get_instance ();
	//port = (gint) gtk_adjustment_get_value (adj);
	gchar* result = gsampler_text_combo_get_active (combo);
	if (result != NULL) 
	{
		port = atoi (string_replace (result, "Port ", ""));
		if (port != self->priv->info->midi_port) 
		{
			if (!gsampler_client_set_channel_midi_port (client, self->priv->info->id, port)) 
			{
				gsampler_main_window_send_message ("warning", 
						                           _("Can't set midi channel port check your device..."));
				gtk_combo_box_set_active (combo, self->priv->info->midi_port);
			}
		}
	}
	g_object_unref (client);
}

static void 
gsampler_channel_on_solo_toggled (GtkToggleButton* button, gpointer data) 
{
	gint state = 1;
	GsamplerClient* client;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (button != NULL);

	if (self->priv->info->solo == TRUE) state = 0;

	client = gsampler_client_get_instance ();
	if (!gsampler_client_set_channel_solo (client, self->priv->info->id, state)) 
	{
		gsampler_main_window_send_message ("warning", _("Can't set channel solo..."));
	}
	g_object_unref (client);
}

static void 
gsampler_channel_on_mute_toggled (GtkToggleButton* button, gpointer data) 
{
	gint state = 1;
	GsamplerClient* client;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (button != NULL);

	if (self->priv->info->mute == TRUE) state = 0;

	client = gsampler_client_get_instance ();
	if (!gsampler_client_set_channel_mute (client, self->priv->info->id, state)) 
	{
		gsampler_main_window_send_message ("warning", _("Can't set channel solo..."));
	}
	g_object_unref (client);
}

static void 
gsampler_channel_on_fx_clicked (GtkButton* button, gpointer data) 
{
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (button != NULL);

	gsampler_main_window_send_message ("info", _("Sorry not yet implemented..."));
}

static void 
gsampler_channel_on_pref_toggled (GtkToggleButton* button, gpointer data) 
{
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (button != NULL);

	if (gtk_toggle_button_get_active (button))
		gtk_widget_show ((GtkWidget*) self->priv->pref_box);
	else
		gtk_widget_hide ((GtkWidget*) self->priv->pref_box);
}

static void
gsampler_channel_on_channel_routing_edited (GtkCellRendererText *cell, const gchar *path,
                 							const gchar *value, GtkTreeStore *store)
{
	GtkTreePath *tree_path = NULL;
	GtkTreeIter iter = {0};
	GsamplerClient *client = NULL;
	g_return_if_fail (path != NULL);
	g_return_if_fail (value != NULL);
	g_return_if_fail (store != NULL);
	
	tree_path = gtk_tree_path_new_from_string (path);
	gtk_tree_model_get_iter ((GtkTreeModel*) store, &iter, tree_path);
	
	gtk_tree_store_set (store, &iter, 1, value, -1);
	gint id;
	gtk_tree_model_get ((GtkTreeModel*) store, &iter, 2, &id,-1);
	client = gsampler_client_get_instance ();
	if (!gsampler_client_set_channel_audio_output_channel(client, id, atoi (path), atoi (value)))
		g_warning ("Can't set channel audio output channel...");
	g_object_unref (client);
	gtk_tree_path_free (tree_path);
}

static GtkWindow *
gsampler_channel_create_routing_window (GsamplerChannel *self, GtkToggleButton* button)
{
	GtkWindow *result;
	GtkVBox *box;
	GtkWidget *title_lbl;
	GtkTreeView *tree_view;
	GtkTreeStore *store = NULL;
	GtkListStore *list = NULL;
	GtkTreeIter iter = {0};
	GsamplerClient *client = NULL;
	int x,y;
	g_return_val_if_fail (self != NULL, NULL);

	GdkScreen *screen = NULL;
	GdkDisplay *disp = gtk_widget_get_display ((GtkWidget*) self);
	gdk_display_get_pointer (disp, &screen, &x, &y, NULL);
	result = (GtkWindow*) gtk_window_new (GTK_WINDOW_POPUP);
	gtk_window_set_screen (result, screen);
	gtk_window_move (result, x+10, y+10);
	box = (GtkVBox*) gtk_vbox_new (FALSE, 5);
	title_lbl = gtk_label_new (NULL);
	gtk_label_set_markup ((GtkLabel*) title_lbl, _("<b>Channel Routing</b>"));
	
	store = gtk_tree_store_new (3, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT);
	gint i;
	gchar** route = g_strsplit (self->priv->info->audio_routing, ",", -1);
	for (i = 0; i < self->priv->info->audio_channels; i++)
	{
		gchar *new_route = NULL;
		if (route[i] != NULL)
			new_route = g_strdup (route[i]);
		else
			new_route = g_strdup_printf ("%d", i);
		gtk_tree_store_append (store, &iter, NULL);
		gtk_tree_store_set (store, &iter, 0, i, 1, new_route, 2, self->priv->info->id, -1);
	}
	g_strfreev (route);

	list = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);
	gint id = gtk_combo_box_get_active (self->priv->audio_combo);
	client = gsampler_client_get_instance ();
	GsamplerDeviceInfo *new_info = gsampler_client_get_device_info (client, id, 0);
	if (new_info->id != (-1))
	{
		for (i = 0; i < new_info->param; i++) 
		{
			gtk_list_store_append (list, &iter);
			gtk_list_store_set (list, &iter, 0, i, 1, g_strdup_printf("%d", i), -1);
		}
	}
	gsampler_device_info_free (new_info);
	g_object_unref (client);
	
	tree_view = (GtkTreeView*) gtk_tree_view_new_with_model ((GtkTreeModel*) store);
	gtk_tree_view_insert_column_with_attributes (tree_view, -1, _("Audio IN"),
	                                             (GtkCellRenderer*) gtk_cell_renderer_text_new (),
	                                             "text", 0, NULL, NULL);
	
	GtkCellRenderer *combo_render = gtk_cell_renderer_combo_new ();
	g_object_set (G_OBJECT (combo_render), 
	              "text-column", 1, 
	              "model", list,
	              "editable", TRUE,
	              "has-entry", FALSE,
	              NULL);
	gtk_tree_view_insert_column_with_attributes (tree_view, -1, _("Audio OUT"),
	                                             (GtkCellRenderer*) combo_render,
	                                             "text", 1, NULL, NULL);
	
	g_signal_connect (G_OBJECT (combo_render), "edited", 
	                  G_CALLBACK (gsampler_channel_on_channel_routing_edited), store);
	
	g_object_unref (store);
	g_object_unref (list);
	gtk_box_pack_start ((GtkBox*) box, (GtkWidget*) title_lbl, TRUE, TRUE,5);
	gtk_box_pack_start ((GtkBox*) box, (GtkWidget*) tree_view, TRUE, TRUE,5);
	gtk_container_add ((GtkContainer*) result, (GtkWidget*) box);
	gtk_widget_show_all ((GtkWidget*) box);
	return result;
}

static void
gsampler_channel_on_channel_routing (GtkToggleButton* button, gpointer data)
{
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	g_return_if_fail (button != NULL);

	if (gtk_toggle_button_get_active (button))
	{
		if (self->priv->chan_routing == NULL) {
			self->priv->chan_routing = gsampler_channel_create_routing_window (self, button);
			gtk_widget_show ((GtkWidget*) self->priv->chan_routing);
		} else
		{
			gtk_widget_show ((GtkWidget*) self->priv->chan_routing);
		}
		
	} else
	{
		if (self->priv->chan_routing != NULL)
		{
			gtk_widget_destroy ((GtkWidget*) self->priv->chan_routing);
			self->priv->chan_routing = NULL;
		}
	}

}

static void 
gsampler_channel_device_count_event (GsamplerClient* client, 
                                     gchar** dev_list, gint type, gpointer data) 
{
	GtkComboBox* tmp_combo = NULL;
	int dev_it;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);
	
	if (type == 0) 
	{
		tmp_combo = g_object_ref (self->priv->audio_combo);
	} else if (type == 1) 
	{
		tmp_combo = g_object_ref (self->priv->midi_combo);
	} else
	{
		g_warning ("Invalide type : %d", type);
		g_object_unref (tmp_combo);
		return;
	}
	gsampler_text_combo_remove_all (tmp_combo);
	gint dev_list_length = g_strv_length (dev_list);
	for (dev_it = 0; dev_it < dev_list_length; dev_it = dev_it + 1) 
	{
		gchar* dev = NULL;
		gint id;
		dev = g_strdup (dev_list[dev_it]);
		id = atoi (dev);
		gsampler_text_combo_insert (tmp_combo, dev, id);
		g_free (dev);		
	}
	
	if (type == 0) 
	{
		gtk_combo_box_set_active ((GtkComboBox*) tmp_combo, self->priv->info->audio);
	} else if (type == 1) 
	{
		gtk_combo_box_set_active ((GtkComboBox*) tmp_combo, self->priv->info->midi);
	}
	g_object_unref (tmp_combo);
}

static void 
gsampler_channel_map_count_event (GsamplerClient* client, gchar** new_list, gpointer data) 
{	
	GtkListStore *list = NULL;
	GsamplerMidiMapInfo *info = NULL;
	GtkTreeIter iter;
	gint map_it;
	gint new_list_length = 0;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_if_fail (self != NULL);

	list = (GtkListStore *) gtk_combo_box_get_model (self->priv->map_combo);
	gtk_list_store_clear (list);
	if (new_list != NULL) new_list_length = g_strv_length (new_list);
	
	for (map_it = 0; map_it < new_list_length; map_it = map_it + 1) 
	{
		gchar* map = NULL;
		gint id;
		id = atoi (new_list[map_it]);
		info = gsampler_client_get_midi_instrument_map_info (client, id);
		map = g_strdup (info->name);
		gtk_list_store_insert (list, &iter, id);
		gtk_list_store_set (list, &iter, 0, id, 1, map, -1);
		g_free (map);
		gsampler_midi_map_info_free (info);
	}
	if (g_strcmp0 (self->priv->info->midi_map, "NONE") != 0)
		gtk_combo_box_set_active ((GtkComboBox*) self->priv->map_combo, atoi (self->priv->info->midi_map));
}

static gboolean
gsampler_channel_on_button_pressed (GtkWidget* sender, GdkEventButton* event, gpointer data) 
{
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_val_if_fail (self != NULL, FALSE);
	
	if (event->button == 1) 
	{
		gsampler_channel_on_load_clicked (self);
	}
	return FALSE;
}

static gboolean
gsampler_channel_on_enter_notify (GtkWidget* sender, GdkEventButton* event, gpointer data)
{
	gsampler_main_window_set_hand_cursor (TRUE);
	return FALSE;
}

static gboolean
gsampler_channel_on_leave_notify (GtkWidget* sender, GdkEventButton* event, gpointer data)
{
	gsampler_main_window_set_hand_cursor (FALSE);
	return FALSE;
}

#if GTK_CHECK_VERSION (2, 90, 0)
static gboolean
gsampler_channel_on_draw_event (GtkWidget* draw_area, cairo_t* context, gpointer data)
{
#else	
static gboolean
gsampler_channel_on_expose_event (GtkWidget* draw_area, GdkEventExpose* event, gpointer data) 
{
	cairo_t* context = gdk_cairo_create (gtk_widget_get_window(draw_area));
#endif
	GdkColor red = {0};
	GdkColor green = {0};
	GdkColor yellow = {0};
	GtkAllocation alloc = {0};
	gdouble xc;
	cairo_pattern_t* pattern;
	PangoFontDescription* font_description;
	PangoLayout* layout;
	gint width = 0;
	gint height = 0;
	gint place1;
	gchar* tmp = NULL;
	gdouble place2;
	GsamplerChannel* self = GSAMPLER_CHANNEL (data);
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (context != NULL, FALSE);
	
	gdk_color_parse ("red", &red);
	gdk_color_parse ("green", &green);
	gdk_color_parse ("yellow", &yellow);
	gtk_widget_get_allocation ((GtkWidget*) draw_area, &alloc);
	xc = (gdouble) ((0 + alloc.width) / 2);
	cairo_set_source_rgba (context, 0, 0, 0, 1);
	cairo_rectangle (context, 0, 0, alloc.width, alloc.height);
	cairo_fill (context);
	pattern = cairo_pattern_create_linear (xc, 0, xc, alloc.height);
	cairo_pattern_add_color_stop_rgba (pattern, 0, 1, 1, 1, 0.4);
	cairo_pattern_add_color_stop_rgba (pattern, 0.9, 0, 0, 0, 0);
	cairo_set_source (context, pattern);
	cairo_rectangle (context, 0, 0, alloc.width, alloc.height);
	cairo_fill (context);
	
	font_description = pango_font_description_from_string (self->priv->font);
	layout = pango_cairo_create_layout (context);
	pango_layout_set_font_description (layout, font_description);
	pango_layout_get_size (layout, &width, &height);
	
	place1 = (alloc.height - (height / PANGO_SCALE)) / 2;
	cairo_set_source_rgb (context, red.red, red.green, red.blue);
	pango_layout_set_text (layout, g_strdup_printf ("Channel %i", self->priv->info->id), -1);
	cairo_move_to (context, 15, place1);
	pango_cairo_update_layout (context, layout);
	pango_cairo_show_layout (context, layout);
	if (self->priv->info->inst_status != 100) 
	{
		if (self->priv->info->inst_status == -1)
			tmp = g_strdup (_("Load an instrument."));
		else 
			tmp = g_strdup_printf ("%i %s", self->priv->info->inst_status, "%");
	} else 
	{
		tmp = g_strdup (self->priv->info->inst_name);
	}
	
	place2 = (gdouble) ((alloc.width / 2) - (strlen (tmp) * 3));
	cairo_set_source_rgb (context, yellow.red, yellow.green, yellow.blue);
	pango_layout_set_text (layout, tmp, -1);
	cairo_move_to (context, place2, place1);
	pango_cairo_update_layout (context, layout);
	pango_cairo_show_layout (context, layout);
	cairo_move_to (context, (alloc.width - 75), place1);
	pango_layout_set_text (layout, self->priv->info->engine, -1);
	cairo_set_source_rgb (context, green.red, green.green, green.blue);
	pango_cairo_update_layout (context, layout);
	pango_cairo_show_layout (context, layout);
	cairo_move_to (context, (alloc.width - 35), place1);
	pango_layout_set_text (layout, g_strdup_printf ("%s / %s", self->priv->stream, self->priv->voice), -1);
	cairo_set_source_rgb (context, yellow.red, yellow.green, yellow.blue);
	pango_cairo_update_layout (context, layout);
	pango_cairo_show_layout (context, layout);
	
	g_free (tmp);
	g_object_unref (layout);
	pango_font_description_free (font_description);
	cairo_pattern_destroy (pattern);
	return FALSE;
}

static GObject * 
gsampler_channel_constructor (GType type, guint n_construct_properties, 
                              GObjectConstructParam * construct_properties) 
{
	GObject * obj;
	GsamplerChannel * self;
	GtkBox* main_box = NULL;
	GtkBox* hbox1 = NULL;
	GtkBox* hbox2 = NULL;
	GtkBox* hbox3 = NULL;
	GtkBox* hbox4 = NULL;
	GtkBox* hbox5 = NULL;
	GtkBox* vbox1 = NULL;
	GtkBox* vbox2 = NULL;
	GtkBox* vbox3 = NULL;
	GtkBox* vbox5 = NULL;
	GtkBox* vbox6 = NULL;
	GtkBox* vbox7 = NULL;
	GtkBox* vbox8 = NULL;
	GtkBox* vbox9 = NULL;
	GtkWidget* vol_img = NULL;
	GtkWidget* pref_img = NULL;
	GtkWidget* close_img = NULL;
	GtkWidget* vol_lbl = NULL;
	GtkWidget* audio_lbl = NULL;
	GtkWidget* midi_lbl = NULL;
	GtkWidget* inst_lbl = NULL;
	GtkWidget* map_lbl = NULL;
	GtkScale* vol_scale = NULL;
	GtkButton* edit_but = NULL;
	GtkButton* close_but = NULL;
	GtkButton* fx_but = NULL;
	GtkToggleButton* cr_but = NULL;
	GtkToggleButton* pref_but = NULL;

	obj = G_OBJECT_CLASS (gsampler_channel_parent_class)->constructor (type, n_construct_properties,
	                                                                   construct_properties);
	self = GSAMPLER_CHANNEL (obj);
	gtk_frame_set_shadow_type ((GtkFrame*) self, GTK_SHADOW_ETCHED_OUT);
	gtk_container_set_border_width ((GtkContainer*) self, 5);

	self->priv->stream = g_strdup ("0");
	self->priv->voice = g_strdup ("0");

	main_box = (GtkBox*) gtk_vbox_new (FALSE, 0);
	hbox1 = (GtkBox*) gtk_hbox_new (FALSE, 0);
	hbox2 = (GtkBox*) gtk_hbox_new (FALSE, 0);
	hbox3 = (GtkBox*) gtk_hbox_new (FALSE, 0);
	hbox4 = (GtkBox*) gtk_hbox_new (FALSE, 5);
	hbox5 = (GtkBox*) gtk_hbox_new (FALSE, 5);
	vbox1 = (GtkBox*) gtk_vbox_new (FALSE, 5);
	vbox2 = (GtkBox*) gtk_vbox_new (FALSE, 0);
	vbox3 = (GtkBox*) gtk_vbox_new (FALSE, 5);
	vbox5 = (GtkBox*) gtk_vbox_new (FALSE, 5);
	vbox6 = (GtkBox*) gtk_vbox_new (FALSE, 5);
	vbox7 = (GtkBox*) gtk_vbox_new (FALSE, 5);
	vbox8 = (GtkBox*) gtk_vbox_new (FALSE, 5);
	vbox9 = (GtkBox*) gtk_vbox_new (FALSE, 5);
	self->priv->solo_but = (GtkToggleButton*) gtk_toggle_button_new_with_label ("Solo");
	gtk_widget_set_name ((GtkWidget *) self->priv->solo_but, "solo");
	self->priv->mute_but = (GtkToggleButton*) gtk_toggle_button_new_with_label ("Mute");
	gtk_widget_set_name ((GtkWidget *) self->priv->mute_but, "mute");
	self->priv->draw_area = (GtkDrawingArea*) gtk_drawing_area_new ();
	vol_img = gtk_image_new_from_stock (GSAMPLER_STOCK_SOUND, GTK_ICON_SIZE_BUTTON);
	vol_lbl = gtk_label_new (NULL);
	gtk_label_set_markup ((GtkLabel*) vol_lbl, _("<b>Volume :</b>"));
	self->priv->vol_adj = (GtkAdjustment *) gtk_adjustment_new (80, 0, 100, 1, 10, 0);
	vol_scale = (GtkScale*) gtk_hscale_new (self->priv->vol_adj);
	gtk_scale_set_value_pos (vol_scale, GTK_POS_RIGHT);
	edit_but = (GtkButton*) gtk_button_new_with_label (_("Edit"));
	close_img = gtk_image_new_from_stock (GTK_STOCK_DELETE, GTK_ICON_SIZE_MENU);
	close_but = (GtkButton*) gtk_button_new ();
	gtk_button_set_image (close_but, close_img);
	fx_but = (GtkButton*) gtk_button_new_with_label (_("Fx"));
	pref_img = gtk_image_new_from_stock (GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU);
	pref_but = (GtkToggleButton*) gtk_toggle_button_new ();
	gtk_button_set_image ((GtkButton*) pref_but, pref_img);
	self->priv->pref_box = (GtkBox*) gtk_vbox_new (FALSE, 0);

	audio_lbl = gtk_label_new (NULL);
	gtk_label_set_markup ((GtkLabel *) audio_lbl, _("<b>Audio Output :</b>"));
	gtk_misc_set_alignment ((GtkMisc *) audio_lbl, 0.00, 0.50);
	midi_lbl = gtk_label_new (NULL);
	gtk_label_set_markup ((GtkLabel *) midi_lbl, _("<b>Midi Input :</b>"));
	gtk_misc_set_alignment ((GtkMisc *) midi_lbl, 0.00, 0.50);
	map_lbl = gtk_label_new (NULL);
	gtk_label_set_markup ((GtkLabel *) map_lbl, _("<b>Midi Map :</b>"));
	gtk_misc_set_alignment ((GtkMisc *) map_lbl, 0.00, 0.50);								  
	inst_lbl = gtk_label_new (NULL);
	gtk_label_set_markup ((GtkLabel *) inst_lbl, _("<b>Instruments :</b>"));
	gtk_misc_set_alignment ((GtkMisc *) inst_lbl, 0.00, 0.50);
	self->priv->midi_combo = gsampler_text_combo_create ();
	self->priv->audio_combo = gsampler_text_combo_create ();
	self->priv->chan_combo = gsampler_text_combo_create ();
	cr_but = (GtkToggleButton*) gtk_toggle_button_new_with_label (_("CR"));
								  
	GtkListStore *map_liste = NULL;
	GtkCellRenderer *cell = NULL;
	map_liste = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);		  
	self->priv->map_combo = (GtkComboBox*) gtk_combo_box_new_with_model (GTK_TREE_MODEL (map_liste));
	cell = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (self->priv->map_combo), cell, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (self->priv->map_combo), cell, "text", 0, NULL);
	cell = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (self->priv->map_combo), cell, FALSE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (self->priv->map_combo), cell, "text", 1, NULL);
								  
	self->priv->inst_combo = gsampler_text_combo_create ();
	self->priv->port_combo = gsampler_text_combo_create ();



	gtk_box_pack_start (vbox1, (GtkWidget*) self->priv->solo_but, FALSE, TRUE, 0);
	gtk_box_pack_start (vbox1, (GtkWidget*) self->priv->mute_but, FALSE, TRUE, 1);
	gtk_box_pack_start (hbox2, vol_img, FALSE, TRUE, 5);
	gtk_box_pack_start (hbox2, vol_lbl, FALSE, TRUE, 5);
	gtk_box_pack_start (hbox2, (GtkWidget*) vol_scale, TRUE, TRUE, 5);
	gtk_box_pack_start (vbox2, (GtkWidget*) self->priv->draw_area, TRUE, TRUE, 0);
	gtk_box_pack_start (vbox2, (GtkWidget*) hbox2, FALSE, TRUE, 5);
	gtk_box_pack_start (vbox3, (GtkWidget*) edit_but, FALSE, TRUE, 0);
	gtk_box_pack_start (vbox3, (GtkWidget*) fx_but, FALSE, TRUE, 1);
	gtk_box_pack_start (hbox1, (GtkWidget*) vbox1, FALSE, TRUE, 2);
	gtk_box_pack_start (hbox1, (GtkWidget*) vbox2, TRUE, TRUE, 2);
	gtk_box_pack_start (hbox1, (GtkWidget*) vbox3, FALSE, TRUE, 2);
	gtk_box_pack_start (vbox5, (GtkWidget*) close_but, FALSE, TRUE, 0);
	gtk_box_pack_start (vbox5, (GtkWidget*) pref_but, FALSE, TRUE, 0);
	gtk_box_pack_start (hbox1, (GtkWidget*) vbox5, FALSE, TRUE, 2);
								  
	gtk_box_pack_start (hbox4, (GtkWidget*) self->priv->audio_combo, TRUE, TRUE, 0);
	gtk_box_pack_start (hbox4, (GtkWidget*) cr_but, FALSE, TRUE, 0);
	gtk_box_pack_start (vbox6, audio_lbl, FALSE, TRUE, 0);
	gtk_box_pack_start (vbox6, (GtkWidget*) hbox4, FALSE, TRUE, 0);
	gtk_box_pack_start (hbox3, (GtkWidget*) vbox6, TRUE, TRUE, 10);

	gtk_box_pack_start (hbox5, (GtkWidget*) self->priv->midi_combo, FALSE, TRUE, 0);
	gtk_box_pack_start (hbox5, (GtkWidget*) self->priv->port_combo, FALSE, TRUE, 0);
	gtk_box_pack_start (hbox5, (GtkWidget*) self->priv->chan_combo, FALSE, TRUE, 0);
	gtk_box_pack_start (vbox7, midi_lbl, FALSE, TRUE, 0);
	gtk_box_pack_start (vbox7, (GtkWidget*) hbox5, FALSE, TRUE, 0);
	gtk_box_pack_start (hbox3, (GtkWidget*) vbox7, TRUE, TRUE, 10);

	gtk_box_pack_start (vbox9, map_lbl, FALSE, TRUE, 0);
	gtk_box_pack_start (vbox9, (GtkWidget*) self->priv->map_combo, FALSE, TRUE, 0);
	gtk_box_pack_start (hbox3, (GtkWidget*) vbox9, TRUE, TRUE, 10);			
								  
	gtk_box_pack_start (vbox8, inst_lbl, FALSE, TRUE, 0);
	gtk_box_pack_start (vbox8, (GtkWidget*) self->priv->inst_combo, FALSE, TRUE, 0);
	gtk_box_pack_start (hbox3, (GtkWidget*) vbox8, TRUE, TRUE, 10);

	gtk_box_pack_start (self->priv->pref_box, (GtkWidget*) hbox3, TRUE, TRUE, 3);
	gtk_box_pack_start (main_box, (GtkWidget*) hbox1, TRUE, TRUE, 0);
	gtk_box_pack_start (main_box, (GtkWidget*) self->priv->pref_box, TRUE, TRUE, 3);
	gtk_container_set_border_width ((GtkContainer*) main_box, 5);
	gtk_container_add ((GtkContainer*) self, (GtkWidget*) main_box);

	gtk_widget_add_events ((GtkWidget*) self->priv->draw_area, GDK_BUTTON_PRESS_MASK | 
	                       GDK_LEAVE_NOTIFY_MASK | GDK_ENTER_NOTIFY_MASK);
	g_signal_connect_object (close_but, "clicked", 
	                         (GCallback) gsampler_channel_on_channel_delete, self, 0);
	g_signal_connect_object (edit_but, "clicked", 
	                         (GCallback) gsampler_channel_on_edit_clicked, self, 0);
	g_signal_connect_object (pref_but, "toggled", 
	                         (GCallback) gsampler_channel_on_pref_toggled, self, 0);
	g_signal_connect_object (fx_but, "clicked", 
	                         (GCallback) gsampler_channel_on_fx_clicked, self, 0);
	g_signal_connect_object (cr_but, "toggled",
	                         (GCallback) gsampler_channel_on_channel_routing, self, 0);
#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 90) || GTK_MAJOR_VERSION < 2
	g_signal_connect_object ((GtkWidget*) self->priv->draw_area, "expose_event", 
	                         (GCallback) gsampler_channel_on_expose_event, self, 0);
#else
	g_signal_connect_object ((GtkWidget*) self->priv->draw_area, "draw", 
	                         (GCallback) gsampler_channel_on_draw_event, self, 0);
	gchar* css = "GtkFrame { border-radius: 5; border-style: solid; border-width: 2; border-color: rgba(0, 0, 0, 0.5);} \
				  #solo:active, #solo:selected { background-image: -gtk-gradient (linear, left top, left bottom, from(#ffff00), to(#ffff00));} \
				  #mute:active, #mute:selected { background-image: -gtk-gradient (linear, left top, left bottom, from(red), to(red));}";
	GtkCssProvider *prov = gtk_css_provider_new ();
	if (gtk_css_provider_load_from_data (prov, css, -1, NULL)) 
	{
		GtkStyleContext *style =  gtk_widget_get_style_context((GtkWidget *) self);
		gtk_style_context_add_provider (style, (GtkStyleProvider *) prov, 
		                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
		style =  gtk_widget_get_style_context((GtkWidget *) self->priv->solo_but);
		gtk_style_context_add_provider (style, (GtkStyleProvider *) prov, 
		                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
		style =  gtk_widget_get_style_context((GtkWidget *) self->priv->mute_but);
		gtk_style_context_add_provider (style, (GtkStyleProvider *) prov, 
		                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	}
	g_object_unref (prov);
#endif

	gtk_widget_show_all ((GtkWidget*) self);
	gtk_widget_hide ((GtkWidget*) self->priv->pref_box);

	return obj;
}

static void 
gsampler_channel_init (GsamplerChannel * self) 
{
	self->priv = GSAMPLER_CHANNEL_GET_PRIVATE (self);
}

static void 
gsampler_channel_finalize (GObject* obj) 
{
	GsamplerChannel * self;
	self = GSAMPLER_CHANNEL (obj);
	gsampler_channel_info_destroy (self->priv->info);
	g_object_unref (self->priv->draw_area);
	g_object_unref (self->priv->instrument);
	g_object_unref (self->priv->pref_box);
	g_object_unref (self->priv->midi_combo);
	g_object_unref (self->priv->audio_combo);
	g_object_unref (self->priv->chan_combo);
	g_object_unref (self->priv->inst_combo);
	g_object_unref (self->priv->vol_adj);
	g_object_unref (self->priv->port_combo);
	g_object_unref (self->priv->solo_but);
	g_object_unref (self->priv->mute_but);
	g_free (self->priv->font);
	if (self->priv->stream != NULL)
		g_free (self->priv->stream);
	if (self->priv->voice != NULL)
		g_free (self->priv->voice);
	G_OBJECT_CLASS (gsampler_channel_parent_class)->finalize (obj);
}

static void 
gsampler_channel_class_init (GsamplerChannelClass * klass) 
{
	gsampler_channel_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GsamplerChannelPrivate));
	G_OBJECT_CLASS (klass)->constructor = gsampler_channel_constructor;
	G_OBJECT_CLASS (klass)->finalize = gsampler_channel_finalize;

	channel_signals[CHANNEL_UPDATE_EVENT] = g_signal_new ("channel_updated",
	                                                      GSAMPLER_TYPE_CHANNEL,
	                                                      G_SIGNAL_RUN_LAST,
	                                                      0, NULL, NULL, 
	                                                      g_cclosure_marshal_VOID__VOID,
	                                                      G_TYPE_NONE, 0);
}

GsamplerChannel* 
gsampler_channel_new (void) 
{
	GsamplerChannel * channel = NULL;
	channel = (GsamplerChannel*) g_object_new (GSAMPLER_TYPE_CHANNEL, NULL);
	return channel;
}

gboolean 
gsampler_channel_active (GsamplerChannel* self, gint id) 
{
	GsamplerClient* client;
	GsamplerOption* option;
	gchar** dev_list;
	gchar** map_list;
	g_return_val_if_fail (self != NULL, FALSE);

	client = gsampler_client_get_instance ();
	
	self->priv->info = gsampler_channel_get_channel_info (self, id);
	if (self->priv->info->id == -1)
	{
		gsampler_channel_info_free (self->priv->info);
		return FALSE;
	}
	
	self->priv->instrument = gsampler_instrument_new ();
	if (self->priv->info->file != NULL && g_strcmp0 (self->priv->info->file, "NONE") !=0)
		gsampler_channel_init_instrument (self);

	gint i = 0;
	while (i < 16) {
		const gchar* temp = NULL;
		i++;
		temp = g_strdup_printf ("Channel %i", i);
		gsampler_text_combo_insert (self->priv->chan_combo, temp, (i - 1));
	}
	gsampler_text_combo_insert (self->priv->chan_combo, "ALL", i);
	if (g_strcmp0 (self->priv->info->midi_chan, "ALL") == 0)
		gtk_combo_box_set_active ((GtkComboBox *) self->priv->chan_combo, 16);
	else
		gtk_combo_box_set_active ((GtkComboBox *) self->priv->chan_combo, atoi(self->priv->info->midi_chan));
	
	gint tmp_port = (gint) gtk_combo_box_get_active (self->priv->port_combo);
	if (tmp_port != self->priv->info->midi_port) 
		gtk_combo_box_set_active (self->priv->port_combo, self->priv->info->midi_port);
	
	option = gsampler_option_get_instance ();
	self->priv->font = gsampler_option_get_string (option, "General", "font");
	
	dev_list = gsampler_client_get_available_audio_output (client);
	gsampler_channel_device_count_event (client, dev_list, 0, self);
	g_strfreev (dev_list);
	dev_list = gsampler_client_get_available_midi_input (client);
	gsampler_channel_device_count_event (client, dev_list, 1, self);
	g_strfreev (dev_list);
	map_list = gsampler_client_get_available_midi_instrument_map (client);
	gsampler_channel_map_count_event (client, map_list, self);
	g_strfreev (map_list);
	gtk_adjustment_set_value (self->priv->vol_adj, self->priv->info->volume * 100);
	gsampler_channel_update_midi_port_combo (self);
	
	g_signal_connect_object (self->priv->vol_adj, "value-changed", 
	                         (GCallback) gsampler_channel_on_volume_changed, self, 0);
	g_signal_connect_object (self->priv->port_combo, "changed", 
	                         (GCallback) gsampler_channel_on_midi_port_changed, self, 0);
	g_signal_connect_object ((GtkComboBox*) self->priv->midi_combo, "changed", 
	                         (GCallback) gsampler_channel_on_midi_changed, self, 0);
	g_signal_connect_object ((GtkComboBox*) self->priv->audio_combo, "changed", 
	                         (GCallback) gsampler_channel_on_audio_changed, self, 0);
	g_signal_connect_object ((GtkComboBox*) self->priv->chan_combo, "changed", 
	                         (GCallback) gsampler_channel_on_midi_channel_changed, self, 0);
	g_signal_connect_object ((GtkComboBox*) self->priv->map_combo, "changed", 
	                         (GCallback) gsampler_channel_on_map_changed, self, 0);
	g_signal_connect_object ((GtkComboBox*) self->priv->inst_combo, "changed", 
	                         (GCallback) gsampler_channel_on_instrument_changed, self, 0);
	g_signal_connect_object (client, "device-count-event", 
	                         (GCallback) gsampler_channel_device_count_event, self, 0);
	g_signal_connect_object (client, "map-count-event",
	                         (GCallback) gsampler_channel_map_count_event, self, 0);
	g_signal_connect_object (self->priv->solo_but, "toggled", 
	                         (GCallback) gsampler_channel_on_solo_toggled, self, 0);
	g_signal_connect_object (self->priv->mute_but, "toggled", 
	                         (GCallback) gsampler_channel_on_mute_toggled, self, 0);
	g_signal_connect_object ((GtkWidget*) self->priv->draw_area, "button-press-event", 
	                         (GCallback) gsampler_channel_on_button_pressed, self, 0);
	g_signal_connect_object ((GtkWidget*) self->priv->draw_area, "leave-notify-event", 
	                         (GCallback) gsampler_channel_on_leave_notify, self, 0);
	g_signal_connect_object ((GtkWidget*) self->priv->draw_area, "enter-notify-event", 
	                         (GCallback) gsampler_channel_on_enter_notify, self, 0);
	g_object_unref (option);
	g_object_unref (client);
	return TRUE;
}

gint 
gsampler_channel_get_channel_id (GsamplerChannel* self) 
{
	gint result = 0;
	g_return_val_if_fail (self != NULL, 0);
	
	result = self->priv->info->id;
	return result;
}

GsamplerChannelInfo * 
gsampler_channel_get_info (GsamplerChannel* self) 
{
	g_return_val_if_fail (self != NULL, NULL);
	
	return gsampler_channel_info_dup (self->priv->info);
}

void 
gsampler_channel_update_voice_count (GsamplerChannel* self, gint val) 
{
	g_return_if_fail (self != NULL);

	self->priv->voice = g_strdup_printf ("%i", val);
	gtk_widget_queue_draw ((GtkWidget*) self->priv->draw_area);
}


void 
gsampler_channel_update_stream_count (GsamplerChannel* self, gint val) 
{
	g_return_if_fail (self != NULL);

	self->priv->stream = g_strdup_printf ("%i", val);
	gtk_widget_queue_draw ((GtkWidget*) self->priv->draw_area);
}

void 
gsampler_channel_update_info (GsamplerChannel* self) 
{
	GsamplerChannelInfo* tmp_info = NULL;
	gchar* old_file;
	g_return_if_fail (self != NULL);
	
	tmp_info = gsampler_channel_get_channel_info (self, self->priv->info->id);
	if (tmp_info != NULL) 
	{
		gsampler_channel_info_destroy (self->priv->info);
		gsampler_channel_info_copy (tmp_info, self->priv->info);

		old_file = gsampler_instrument_get_file (self->priv->instrument);
		if (g_strcmp0 (self->priv->info->file, old_file) != 0 &&
		    self->priv->info->inst_status == 100) 
		{
			gsampler_channel_init_instrument (self);
			gsampler_main_window_set_wait_cursor (FALSE);
		}
		g_free (old_file);
		if (gtk_combo_box_get_active ((GtkComboBox*) self->priv->audio_combo) != self->priv->info->audio) 
		{
			gtk_combo_box_set_active ((GtkComboBox*) self->priv->audio_combo, self->priv->info->audio);
		}
		if (gtk_combo_box_get_active ((GtkComboBox*) self->priv->midi_combo) != self->priv->info->midi) 
		{
			gtk_combo_box_set_active ((GtkComboBox*) self->priv->midi_combo, self->priv->info->midi);
		}
		gsampler_channel_update_midi_port_combo (self);
		/*gint tmp_port = (gint) gtk_adjustment_get_value (self->priv->port_adj);
		if (tmp_port != self->priv->info->midi_port) 
		{
			gtk_adjustment_set_value (self->priv->port_adj, (gdouble) self->priv->info->midi_port);
		}*/
		gint active = gtk_combo_box_get_active (self->priv->chan_combo);
		if (active >= 16 || active == -1)
		{
			gtk_combo_box_set_active ((GtkComboBox*) self->priv->chan_combo, 16);
		} else
		{
			gint id = atoi (self->priv->info->midi_chan);
			gtk_combo_box_set_active ((GtkComboBox*) self->priv->chan_combo, id);
		}
		gtk_widget_queue_draw ((GtkWidget*) self->priv->draw_area);
		gsampler_channel_info_free (tmp_info);
		g_signal_emit (self, channel_signals[CHANNEL_UPDATE_EVENT],0);
	}
}